SDKP_DEFAULTS = {
    profile = {
        trackloot = true,
        lootfilter = { 
			["ff1eff00"] = false,	-- Uncommon
			["ff0070dd"] = true,	-- Rare
			["ffa335ee"] = true,	-- Epic
			["ffff8000"] = true,	-- Legendary
        },
        ignoreloot = {
			["Sunmote"] = true,
			["Lionseye"] = true,
			["Badge of Justice"] = true,
			["Pyrestone"] = true,
			["Crimson Spinel"] = true,
			["Seaspray Emerald"] = true,
			["Void Crystal"] = true,
			["Empyrean Sapphire"] = true,
			["Shadowsong Amethyst"] = true,
			["Large Prismatic Shard"] = true,
        },
        debug = false,
        snapshots = {},
    	waitlist = {},
    	bids = {},
    	loot = {},
    	
    	checkattendance = true,
    	relationaldkp = false,
		initialgearvalue = 0,
		announceloot = true,
		taxrate = .30,
		taxthreshold = 30,
		attchecklen = 180, -- 3 min
		sanctionlen = 240,
    },
};

SDKP_OPTIONS = {
    name = "Sachmo's Dkp Mod",
    type = 'group',
    handler = sdkp,
    childGroups = "tab",
    args = {
    	
    	-- Main options tab.
    	main = {
    		name = "Main",
    		type = "group",
    		cmdInline = true,
    		order = 1,
    		args = {
    		   	checkatt = {
		   			name = "checkatt",
		   			type = "execute",
		   			desc = "Require all people in the attendance list to report in.",
		   			func = "CmdCheckAtt",
		   			guiHidden = true,
		   			order = 1,
		   		},
    	   		help = {
		   			name = "help",
		   			type = "execute",
		   			desc = "Prints usage info",
		   			func = function() LibStub("AceConfigCmd-3.0").HandleCommand(sdkp, "sdkp", "sdkp", ""); end,
		   			guiHidden = true,
		   			order = 1,
		   		},
		    	show = {
		   			name = "show",
		   			type = "execute",
		   			desc = "Show the bidding window",
		   			func = function() sdkp:ShowBids(); end,
		   			guiHidden = true;
					order = 2,
		   		},
		   		sync = {
		   			name = "sync",
		   			type = "input",
		   			desc = "syncs data with player",
		   			set = "CmdSync",
		   			guiHidden = true;
					order = 3,
		   		},		   	
		    	
		    	snapshotdata = {
		    		name = "Snapshot Data",
		    		type = "group", 
		    		inline = true,
		    		order = 5, 
		    		args = {
			    		purge = {
				    		name = "Purge",
				    		type = "execute",
				    		desc = "Purges snapshot and loot data.",
				    		func = "Purge",
				    		confirm = function() return true end, -- AceConfigConsole barfs on boolean type.
				    		confirmText = "Are you sure you want to purge all snapshot and loot data?",
				    		order = 5,
				    		width = "half",
				    	},
				    	export = {
				   			name = "Export",
				   			type = "execute",
				   			desc = "Export Snapshot Data",
				   			func = "CmdExport",
							order = 6,
							width = "half",
				   		},
		    		},
		    	},		
		    	
		    	misc = {
		    	   	name = "Misc",
		    		type = "group", 
		    		inline = true,
		    		order = 10, 
		    		args = { 	
		    		    debug = {
				    		name = "Debug",
				    		type = "toggle",
				    		desc = "Print debug info",
				    		get = "IsDebug",
				    		set = "ToggleDebug",
				    		order = 1,
				    	},
				    },
			    },
    		},
    	},
    	
    	-- Loot options tab.
    	loot = {
    		name = "Loot",
    		type = "group",
    		cmdInline = true,
    		order = 2,
    		args = {
    			trackloot = {
		    		name = "Track loot",
		    		type = "toggle",
		    		desc = "On/Off Tracking of loot.",
		    		get = "IsTrackLoot",
					set = "ToggleTrackLoot",
					width = "full",
					order = 0,												 
		    	},
		    	announceloot = {
		    		name = "Announce loot",
		    		type = "toggle",
		    		desc = "On/Off Announcing of loot in guild chat, if tracking loot.",
		    		get = "IsAnnounceLoot",
					set = "ToggleAnnounceLoot",
					width = "full",
					disabled = function(info) return not sdkp:IsTrackLoot(info) end,
					order = 1,												 
		    	},
		    	taxthreshold = {
		    		name = "Tax threshold", 
		    		type = "range",
		    		min = 0,
		    		max = 100,
		    		step = 1,
		    		bigStep = 1,
		    		desc = "If a player's dkp is greater than this amount, players will be taxed on purchases.",
		    		get = "GetTaxThreshold",
					set = "SetTaxThreshold",
					width = "full",
					disabled = function(info) return not sdkp:IsTrackLoot(info) end,
					order = 5,												 
		    	},
		    	taxrate = {
		    		name = "Tax rate", 
		    		type = "range",
		    		min = 0,
		    		max = 1,
		    		step = 0.01,
		    		bigStep = 0.01,
		    		isPercent = true,
		    		desc = "Purchases will be taxed at this rate if the threshold is met.",
		    		get = "GetTaxRate",
					set = "SetTaxRate",
					width = "full",
					disabled = function(info) return not sdkp:IsTrackLoot(info) end,
					order = 6,												 
		    	},
		    	lootfilter = {
		    		name = "Track items of these qualities:",
		    		type = "multiselect",
		    		desc = "Track this type of loot, if tracking loot.",
		    		values = {
						"Uncommon",
						"Rare",
						"Epic",
						"Legendary",
					},
		    		get = "GetLootFilter",
		    		set = "SetLootFilter",
		    		disabled = function(info) return not sdkp:IsTrackLoot(info) end,
		    		order = 9,
		    	},
		    	-- Toggle tracking of loot  	
		    	ignoreloot = {
		    		name = "Ignore loot",
		    		type = "input",
		    		desc = "alternate: |cffffff78il|r - Toggle loot from ignore list.    (ex: /sdkp il |cff0070dd|Hitem:32428:0:0:0:0:0:0:0|h[Heart of Darkness]|h|r)",
		    		set = "CmdIgnoreLoot",
		    		guiHidden = true,
		    	},
		    	il = {
		    		name = "il",
		    		type = "input",
		    		set = "CmdIgnoreLoot",
		    		hidden = true,
		    	},
    		},
    	},
    	
    	-- Dkp system options tab.
    	dkpsystem = {
    		name = "Dkp",
    		type = "group",
    		cmdInline = true,
    		order = 3,
    		args = {
    			checkattendance = {
		    		name = "Enable attendance checks",
		    		type = "toggle",
		    		desc = "Initiate a guild attendance check when taking a snapshot. (Only happens if tracking loot)",
		    		get = "IsAttCheck",
					set = "ToggleAttCheck",
					width = "full",
					disabled = function(info) return not sdkp:IsTrackLoot(info) end,
					order = 0,												 
		    	},
		    	attchecklen = {
		    		name = "Attendance check length", 
		    		type = "range",
		    		min = 10,
		    		max = 6*60,
		    		step = 1,
		    		bigStep = 5,
		    		desc = "How long an attendance check will remain active. (time in seconds)",
		    		get = "GetAttCheckLen",
					set = "SetAttCheckLen",
					width = "full",
					disabled = function(info) return not (sdkp:IsTrackLoot(info) and sdkp:IsAttCheck(info)) end,
					order = 1,												 
		    	},
		    	sanctionlen = {
		    		name = "Whisper bot sanction penalty", 
		    		type = "range",
		    		min = 10,
		    		max = 6*60,
		    		step = 1,
		    		bigStep = 5,
		    		desc = "How long a sanction will remain active. (time in seconds)",
		    		get = "GetSanctionLen",
					set = "SetSanctionLen",
					width = "full",
					disabled = function(info) return not (sdkp:IsTrackLoot(info) and sdkp:IsAttCheck(info)) end,
					order = 2,												 
		    	},
		    		
    			relationaldkp = {
		    		name = "Relational Dkp",
		    		type = "group", 
		    		inline = true,
		    		order = 10, 
		    		args = {
				    	userelationaldkp = {
				    		name = "Use Relational Dkp",
				    		type = "toggle",
				    		desc = "Select to enable use of relational dkp system.",
				    		get = "IsRelationalDkp",
							set = "ToggleRelationalDkp",
							order = 0,												 
				    	},
				    	initialgearvalue = {
				    		name = "Initial Gear Value",
				    		type = "input",
				    		usage = "Must be a number.",
				    		pattern = "%d+%.?%d*",
				    		desc = "This number is added to the spent dkp of a player in the relational dkp system",
				    		get = "GetInitialGearValue",
							set = "SetInitialGearValue",
							multiline = false;
							disabled = function(info) return not sdkp:IsRelationalDkp(info); end,
							order = 1,												 
				    	},
		    		},
		    	},		
    		},
    	},
    	
    	-- Manually assign loot to a player  	
    	assignloot = {
    		name = "Ignore loot",
    		type = "input",
    		desc = "alternate: |cffffff78al|r - Manually assign loot to a player (ex: /sdkp al |cff0070dd|Hitem:32428:0:0:0:0:0:0:0|h[Heart of Darkness]|h|r Sachmo)",
    		set = "CmdAssginLoot",
    		guiHidden = true,
    	},
    	al = {
    		name = "il",
    		type = "input",
    		set = "CmdAssignLoot",
    		hidden = true,
    	},
    	
    	-- Manually assign noobery to a player  	
    	noob = {
    		name = "Noobery",
    		type = "input",
    		desc = "Manually assign Minor Noobery penalty to a player (ex: /sdkp minornoob Sachmo)",
    		set = function(info, player) sdkp:CmdNoobery("minor", player) end,
    		guiHidden = true,
    	},
    	
    	-- Manually assign major noobery to a player  	
    	majornoob = {
    		name = "Major Noobery",
    		type = "input",
    		desc = "Manually assign Major Noobery penalty to a player (ex: /sdkp majornoob Sachmo)",
    		set = function(info, player) sdkp:CmdNoobery("major", player) end,
    		guiHidden = true,
    	},
    	
    	-- Manually add/remove to snapshot
    	ss = {
    		name = "ss",
    		desc = "Force a snapshot, or add/remove people form the last one. [ /sdkp ss ] for more info.",
    		type = "group",
    		order = 7,
    		guiHidden = true,
    		args = {
	    		add = {
	    			name = "ss_add",
	    			desc = "Add player to the last snapshot.    (ex: /sdkp ss add Sachmo)",
	    			type = "input",
	    			set = function (info, player)
	    				local snapshots = sdkp.db.profile.snapshots;
	    				if snapshots and snapshots[#snapshots] then 
	    					sdkp:CmdHandleManualSnapshot(player, "add", snapshots[#snapshots]);
	    				else
	    					sdkp:Print("no snapshots found to add to!");
	    				end
	    			end,
	    			guiHidden = true,
	    			order = 1,
	    		},
	    		remove = {
	    			name = "ss_remove",
	    			desc = "Remove player from the last snapshot.    (ex: /sdkp ss remove Sachmo)",
	    			type = "input",
	    			set = function (info, player)
	    				local snapshots = sdkp.db.profile.snapshots;
	    				if snapshots and snapshots[#snapshots] then 
	    					sdkp:CmdHandleManualSnapshot(player, "remove", snapshots[#snapshots]);
	    				else
	    					sdkp:Print("no snapshots found to remove from!");
	    				end
	    			end,
	    			guiHidden = true,
	    			order = 2,
	    		},
	    		force = {
	    			name = "ss_force",
	    			desc = "force a snapshot to be taken.  (ex: /sdkp ss force)",
	    			type = "execute",
	    			func = "CmdSnapshot",
	    			guiHidden = true,	  
	    			order = 3,  		
	    		},
		    }, -- args
	    }, -- ss
	    st = {
	    	name = "st",
	    	desc = "st",
	    	type = "execute",
		   	desc = "Show the bidding window",
		   	func = "ST",
		   	guiHidden = true;
			order = 10,
	    }
    }, -- args
};

SDKP_COLORS = {
	"ff1eff00", -- [1]
	"ff0070dd", -- [2]
	"ffa335ee", -- [3]
	"ffff8000", -- [4]
};
SDKP_ICOLORS = {
	["ff1eff00"] = 1,
	["ff0070dd"] = 2,
	["ffa335ee"] = 3,
	["ffff8000"] = 4,
};

SDKP_FILTERS_LIST = {
	"Show All", -- [1]
	"  by class - Deathknight", -- [2]
	"  by class - Druid", -- [3]
	"  by class - Hunter", -- [4]
	"  by class - Mage", -- [5]
	"  by class - Paladin", -- [6]
	"  by class - Priest", -- [7]
	"  by class - Rogue", -- [8]
	"  by class - Shaman", -- [9]
	"  by class - Warlock", -- [10]
	"  by class - Warrior", -- [11]
	"Only Raid", -- [12]
};
SDKP_FILTERS = {
	[SDKP_FILTERS_LIST[1]] = function(self, row)
		return true;
	end, -- [1]
	[SDKP_FILTERS_LIST[12]] = function(self, row)
		local nRaidMembers = GetNumRaidMembers();
		for i = 1, nRaidMembers do
			local raidMember = GetRaidRosterInfo(i);
			if row.cols[1].value == raidMember then 
				return true;
			end
		end
		return false;
	end, -- [2]
};
for i = 2, 11 do 
	SDKP_FILTERS[SDKP_FILTERS_LIST[i]] = function(self, row)
		local _, _, class = string.find(SDKP_FILTERS_LIST[i], "%s([^%s]+)$");
		local color = row.cols[1].color;
		if type(color) == "function" then 
			color = color(unpack(row.cols[1].colorargs or {}));
		end
		if color == RAID_CLASS_COLORS[class:upper()] then 
			return SDKP_FILTERS[SDKP_FILTERS_LIST[1]](self, row);
		else 
			return false;
		end
	end
end

SDKP_RAID_ZONE = {
	["Black Temple"] = true,
	["Sunwell Plateau"] = true,
	["Naxxramas"] = true,
	["The Obsidian Sanctum"] = true,
	["The Eye of Eternity"] = true,
	["Ulduar"] = true,
};

SDKP_NOUPDATE_ZONE = { 
	-- this is mostly to prevent the scrolling table from adding ppl in bg raids.
	["Alterac Valley"] = true,
	["Warsong Gulch"] = true,
	["Arathi Basin"] = true,
	["Eye of the Storm"] = true,
};

SDKP_UNIT_TO_ENCOUNTER = {
	-- Testing SM
	["Scarlet Commander Mograine"] = "Scarlet Monastery Cathedral",
	["High Inquisitor Whitemane"] = "Scarlet Monastery Cathedral",
	
	-- BT
	-- single quotes fuck up the import( wont track loot as single raid for boss )... SIGH
	["High Warlord Naj'entus"] = "High Warlord Najentus", 
	["Gathios the Shatterer"] = "The Illidari Council",
	["High Nethermancer Zerevor"] = "The Illidari Council",
	["Lady Malande"] = "The Illidari Council",
	["Veras Darkshadow"] = "The Illidari Council",
	["Essence of Anger"] = "Reliquary of Souls",
	
	-- Sunwell
	["Sathrovarr the Corruptor"] = "Kalecgos",
	["Lady Sacrolash"] = "Eredar Twins",
	["Grand Warlock Alythess"] = "Eredar Twins",
	["Entropius"] = "Muru", -- no single quote pls..
	["Kil'jaeden"] = "Kiljaeden", -- no single quote pls..
	
	--Naxxramus
	["Anub'Rekhan"] = "AnubRekhan",
	["Thane Korth'azz"] = "The Four Horsemen",
	["Lady Blaumeux"] = "The Four Horsemen",
	["Highlord Mograine"] = "The Four Horsemen",
	["Sir Zeliek"] = "The Four Horsemen",
	["Kel'Thuzad"] = "KelThuzad",
};

SDKP_UNIT_BOSS_IGNORE = {
	["Shadron"] = true,
	["Tenebron"] = true,
	["Vesperon"] = true,
};